package org.fuseri.modulecertificate;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.certificate.CertificateCreateDto;
import org.fuseri.model.dto.certificate.CertificateDto;
import org.fuseri.model.dto.course.CourseDto;
import org.fuseri.model.dto.course.LanguageTypeDto;
import org.fuseri.model.dto.course.ProficiencyLevelDto;
import org.fuseri.model.dto.user.AddressDto;
import org.fuseri.model.dto.user.UserDto;
import org.junit.jupiter.api.Test;
import org.springdoc.core.converters.models.Pageable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;


@SpringBootTest
@AutoConfigureMockMvc
class CertificateControllerTests {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private MockMvc mockMvc;

    private final UserDto USER = new UserDto("novakovat",
            "novakova@gamil.com", "Tereza", "Nováková", new AddressDto());
    private final CourseDto COURSE = new CourseDto("AJ1", 10,
            LanguageTypeDto.ENGLISH, ProficiencyLevelDto.A1);

    @Test
    void generateCertificate() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(USER, COURSE)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    @Test
    void generateCertificateWithNullUser() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(null, COURSE)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void generateCertificateWithNullCourse() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(USER, null)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void generateCertificateWithoutParams() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificate() throws Exception {
        String response = mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(USER, COURSE)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()).andReturn().getResponse().getContentAsString();

        Long id = objectMapper.readValue(response, CertificateDto.class).getId();

        mockMvc.perform(get("/certificates/find").param("id", id.toString()))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(id));
    }

    @Test
    void findCertificateWithoutId() throws Exception {
        mockMvc.perform(get("/certificates/find"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificatesForUser() throws Exception {
        mockMvc.perform(get("/certificates/findForUser").param("userId", "0"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isNotEmpty());
    }

    @Test
    void findCertificatesWithoutUserId() throws Exception {
        mockMvc.perform(get("/certificates/findForUser"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdForUserAndCourse() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("userId", "0")
                        .param("courseId", "0"))
                .andExpect(status().isOk())
                .andExpect(content().string("[]"));
    }

    @Test
    void findCertificateIdWithoutUserId() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("courseId", "0"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdWithoutCourseId() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse")
                        .param("userId", "0"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void findCertificateIdWithoutParams() throws Exception {
        mockMvc.perform(get("/certificates/findForUserAndCourse"))
                .andExpect(status().is4xxClientError());
    }

    @Test
    void deleteCertificate() throws Exception {
        mockMvc.perform(delete("/certificates/delete")
                        .param("id", "0"))
                .andExpect(status().isOk());
    }

    @Test
    void deleteCertificateWithoutParam() throws Exception {
        mockMvc.perform(delete("/certificates/delete"))
                .andExpect(status().is4xxClientError());
    }


    @Test
    void findAllCertificates() throws Exception {
        mockMvc.perform(post("/certificates/generate")
                        .content(asJsonString(new CertificateCreateDto(USER, COURSE)))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());

        mockMvc.perform(get("/certificates/findAll")
                .content("{ \"page\": 0, \"size\": 1, \"sort\": []}")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }

    @Test
    void findAllCertificatesWithoutParam() throws Exception {
        mockMvc.perform(get("/certificates/findAll"))
                .andExpect(status().is4xxClientError());
    }

    private static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
