package org.fuseri.modulelanguageschool.lecture;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import org.fuseri.model.dto.lecture.LectureCreateDto;
import org.fuseri.model.dto.lecture.LectureDto;
import org.fuseri.model.dto.user.UserDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * This class represents a RESTful controller for Lecture resources.
 * It handles incoming HTTP requests related to lectures, and delegates them to the appropriate service method.
 */
@Api(tags = "Lecture Controller")
@RestController
@RequestMapping("/lectures")
public class LectureController {

    private final LectureFacade lectureFacade;

    @Autowired
    public LectureController(LectureFacade lectureFacade) {
        this.lectureFacade = lectureFacade;
    }


    /**
     * Creates a new lecture resource by delegating to the LectureService's create method.
     *
     * @param lecture the LectureDto representing the lecture to be created
     * @return the LectureDto representing the newly created lecture
     */
    @ApiOperation(value = "Create a new lecture")
    @PostMapping
    public LectureDto create(@Valid @RequestBody LectureCreateDto lecture) {
        return lectureFacade.create(lecture);
    }

    /**
     * Retrieves a lecture resource by its ID.
     *
     * @param courseId the ID of the lecture to find
     * @return the LectureDto representing the found lecture
     */
    @ApiOperation(value = "Retrieve a lecture by its ID")
    @GetMapping("find/{courseId}")
    public LectureDto find(@PathVariable Long courseId) {
        return lectureFacade.findById(courseId);
    }

    /**
     * Retrieves a list of lectures for the corresponding course
     *
     * @param courseId the course to retrieve lectures from
     * @return the list of LectureDtos
     */
    @ApiOperation(value = "Retrieve a list of lectures for the corresponding course")
    @GetMapping("/findByCourse")
    public List<LectureDto> findByCourse(@Valid @RequestParam Long courseId) {
        return lectureFacade.findAll(courseId);
    }

    /**
     * Updates an existing lecture resource by delegating to the LectureService's update method.
     *
     * @param lecture the CourseCreateDto representing the updated lecture
     * @return the LectureDto representing the updated lecture
     */
    @ApiOperation(value = "Update an existing lecture")
    @PutMapping("/update/{id}")
    public LectureDto update(@PathVariable Long id, @Valid @RequestBody LectureCreateDto lecture) {
        return lectureFacade.update(id, lecture);
    }

    /**
     * Deletes a lecture resource by its ID.
     *
     * @param id the ID of the lecture to delete
     */
    @ApiOperation(value = "Delete a lecture by its ID")
    @DeleteMapping("/delete/{id}")
    public void delete(@PathVariable Long id) {
        lectureFacade.delete(id);
    }


    /**
     * Adds lecturer to the existing lecture resource
     *
     * @param id          id of lecture to update
     * @param lecturerDto UserDto for the course lecturer
     * @return the LectureDto representing the updated lecture
     */
    @ApiOperation(value = "Add lecturer to the existing lecture")
    @PatchMapping("/setLecturer/{id}")
    public LectureDto setLecturer(@PathVariable Long id, @RequestBody UserDto lecturerDto) {
        return lectureFacade.setLecturer(id, lecturerDto);
    }

    /**
     * Adds student to the existing lecture resource
     *
     * @param id      id of lecture to update
     * @param student UserDto for the course student
     * @return the LectureDto representing the updated lecture
     */
    @ApiOperation(value = "Add student to the existing lecture")
    @PatchMapping("/enrol/{id}")
    public LectureDto enrol(@PathVariable Long id, @RequestBody UserDto student) {
        return lectureFacade.enrol(id, student);
    }

    /**
     * Removes student from the existing lecture resource
     *
     * @param id      id of lecture to update
     * @param student UserDto for the course student
     * @return the LectureDto representing the updated lecture
     */
    @ApiOperation(value = "Remove student from the existing lecture")
    @PatchMapping("/expel/{id}")
    public LectureDto expel(@PathVariable Long id, @RequestBody UserDto student) {
        return lectureFacade.expel(id, student);
    }
}